SystemOrganization addCategory: #'GPS-Core'!
SystemOrganization addCategory: #'GPS-Test'!
SystemOrganization addCategory: #'GPS-Morphic'!

Object subclass: #GPSAngle
	instanceVariableNames: 'radians'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Core'!

!GPSAngle class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 13:25'!
degrees: degreesNumber 
	"answer a new instance of the receiver representing the"
	^ self radians: degreesNumber degreesToRadians! !

!GPSAngle class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 13:26'!
degrees: degreesNumber minutes: minutesNumber 
	"answer a new instance of the receiver representing the"
	^ self degrees: (degreesNumber + (minutesNumber / 60)) ! !

!GPSAngle class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 13:27'!
degrees: degreesNumber minutes: minutesNumber seconds: secondsNumber 
	"answer a new instance of the receiver representing the"
	^ self degrees: degreesNumber minutes: (minutesNumber ) + (secondsNumber / 60)! !

!GPSAngle class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 14:37'!
fromString: aString 
	"answer a new instance of the receiver based on a string with  
	format ddmm.mmmm"
	" 
	GPSAngle fromString: '3855.1939'.
	"
	| number degrees minutes |
	number := aString asNumber.
	degrees := (number / 100) truncated.
	minutes := number - (degrees * 100).
	""
	^ self degrees: degrees minutes: minutes! !

!GPSAngle class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 13:25'!
radians: radiansNumber 
	"answer a new instance of the receiver representing the"
	^ self new initializeRadians: radiansNumber! !

!GPSAngle methodsFor: 'comparing' stamp: 'dgd 7/7/2005 13:22'!
= anObject 
	"answer whether the receiver and the argument represent the  
	same object."
	self class = anObject class
		ifFalse: [^ false].
	^ self radians  = anObject radians ! !

!GPSAngle methodsFor: 'accessing' stamp: 'dgd 7/7/2005 13:05'!
degrees
	"answer the receiver's degrees"
	^ radians radiansToDegrees! !

!GPSAngle methodsFor: 'accessing' stamp: 'dgd 7/7/2005 13:38'!
degreesMinutesSeconds
	"answer the receiver's degrees, minutes and seconds"
	| degrees minutes seconds |
	degrees := self degrees truncated.
	minutes := (self degrees - degrees * 60) truncated.
	seconds := self degrees - degrees - (minutes / 60) * 3600.
	""
	^ {degrees. minutes. seconds}! !

!GPSAngle methodsFor: 'comparing' stamp: 'dgd 7/7/2005 13:18'!
hash
	^ radians hash! !

!GPSAngle methodsFor: 'initialization' stamp: 'dgd 7/7/2005 13:45'!
initializeRadians: aNumber 
	"initialize the receiver's radians"
	radians := aNumber.
self normalize! !

!GPSAngle methodsFor: 'arithmetic' stamp: 'dgd 7/7/2005 14:36'!
negated
	"answer an angle that is the negation of the receiver"
	^ self class radians: self radians negated! !

!GPSAngle methodsFor: 'initialization' stamp: 'dgd 7/7/2005 14:31'!
normalize
	"normalize the receiver into the valid range"
	| range |
	range := self range.
	""
	[radians > range second]
		whileTrue: [radians := radians - (Float pi * 2)].
	""
	[radians < range first]
		whileTrue: [radians := radians + (Float pi * 2)] ! !

!GPSAngle methodsFor: 'printing' stamp: 'dgd 7/7/2005 13:58'!
printDegreesMinutesSecondsOn: aStream 
	| degreesMinutesSeconds |
	degreesMinutesSeconds := self degreesMinutesSeconds.
	aStream 
		 nextPutAll: degreesMinutesSeconds first asString;
		 nextPutAll: ' ';
		 nextPutAll: degreesMinutesSeconds second asString;
		 nextPutAll: ''' ';
		 nextPutAll: (degreesMinutesSeconds third roundTo: 0.01) asString;
		 nextPutAll: '"'! !

!GPSAngle methodsFor: 'printing' stamp: 'dgd 7/14/2005 11:45'!
printOn: aStream 
	aStream nextPutAll: ' ';
		 nextPutAll: self degrees asString;
		 nextPutAll: ' ('.
	self printDegreesMinutesSecondsOn: aStream.
	aStream 
		 nextPutAll: ')'.! !

!GPSAngle methodsFor: 'accessing' stamp: 'dgd 7/7/2005 13:01'!
radians
	"answer the receiver's radians"
	^ radians! !

!GPSAngle methodsFor: 'initialization' stamp: 'dgd 7/7/2005 14:30'!
range
	"answer the receiver's valid range"
	^ {0. Float pi * 2}! !

GPSAngle subclass: #GPSLatitude
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Core'!

!GPSLatitude class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 14:40'!
fromString: aString nsIndicator: nsIndicator 
	"answer a new instance of the receiver based on a string with  
	format ddmm.mmmm"
	" 
	GPSLatitude fromString: '3855.1939' nsIndicator: 'N'. 
	GPSLatitude fromString: '3855.1939' nsIndicator: 'S'.
	"
	| result |
	result := self fromString: aString.
	nsIndicator = 'N'
		ifFalse: [result := result negated].
	^ result! !

!GPSLatitude methodsFor: 'printing' stamp: 'dgd 7/7/2005 14:00'!
printOn: aStream 
	self printDegreesMinutesSecondsOn: aStream.
aStream nextPutAll: (radians positive ifTrue:[' North'] ifFalse:[' South'])! !

!GPSLatitude methodsFor: 'initialization' stamp: 'dgd 7/7/2005 14:30'!
range
	"answer the receiver's valid range"
	^ {Float pi negated. Float pi}! !

GPSAngle subclass: #GPSLongitude
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Core'!

!GPSLongitude class methodsFor: 'instance creation' stamp: 'dgd 7/11/2005 12:02'!
fromString: aString ewIndicator: ewIndicator 
	"answer a new instance of the receiver based on a string with  
	format ddmm.mmmm"
	" 
	GPSLongitude fromString: '3855.1939' ewIndicator: 'E'.
	GPSLongitude fromString: '3855.1939' ewIndicator: 'W'.
	"
	| result |
	result := self fromString: aString.
	ewIndicator = 'E'
		ifFalse: [result := result negated].
	^ result! !

!GPSLongitude methodsFor: 'printing' stamp: 'dgd 7/11/2005 12:02'!
printOn: aStream 
	self printDegreesMinutesSecondsOn: aStream.
	aStream
		nextPutAll: (radians positive
				ifTrue: [' East']
				ifFalse: [' West'])! !

!GPSLongitude methodsFor: 'initialization' stamp: 'dgd 7/11/2005 12:02'!
range
	"answer the receiver's valid range"
	^ {Float pi negated. Float pi}! !

Object subclass: #GPSSatellite
	instanceVariableNames: 'id elevation azimuth snr signalToNoiseRation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Core'!

!GPSSatellite class methodsFor: 'instance creation' stamp: 'dgd 7/7/2005 12:04'!
id: idString elevation: elevationString azimuth: azimuthString signalToNoiseRation: signalToNoiseRationString 
	^ self new
		initializeID: idString
		elevation: elevationString
		azimuth: azimuthString
		signalToNoiseRation: signalToNoiseRationString! !

!GPSSatellite methodsFor: 'accessing' stamp: 'dgd 7/7/2005 12:01'!
azimuth
	"answer the receiver's azimuth"
	^ azimuth! !

!GPSSatellite methodsFor: 'accessing' stamp: 'dgd 7/7/2005 12:01'!
elevation
	"answer the receiver's elevation"
	^ elevation! !

!GPSSatellite methodsFor: 'accessing' stamp: 'dgd 7/7/2005 12:00'!
id
	"answer the receiver's id"
	^ id! !

!GPSSatellite methodsFor: 'initialization' stamp: 'dgd 7/7/2005 12:03'!
initializeID: idString elevation: elevationString azimuth: azimuthString signalToNoiseRation: signalToNoiseRationString 
	id := idString asNumber.
	elevation := elevationString asNumber.
	azimuth := azimuthString asNumber.
	signalToNoiseRation := signalToNoiseRationString asNumber! !

!GPSSatellite methodsFor: 'printing' stamp: 'dgd 7/7/2005 12:03'!
printOn: aStream 
	super printOn: aStream.
	""
	aStream nextPutAll: ' (id: ';
		 nextPutAll: id asString;
		 nextPutAll: ', elevation: ';
		 nextPutAll: elevation asString;
		 nextPutAll: ', azimuth: ';
		 nextPutAll: azimuth asString;
		 nextPutAll: ', signalToNoiseRation: ';
		 nextPutAll: signalToNoiseRation asString;
		 nextPutAll: ')'! !

!GPSSatellite methodsFor: 'accessing' stamp: 'dgd 7/7/2005 12:03'!
signalToNoiseRation
	"answer the receiver's signalToNoiseRation"
	^ signalToNoiseRation! !

RectangleMorph subclass: #GPSMorph
	instanceVariableNames: 'port gps'
	classVariableNames: 'DisconnectedForm ConnectedForm'
	poolDictionaries: ''
	category: 'GPS-Morphic'!

!GPSMorph class methodsFor: 'scripting' stamp: 'dgd 7/14/2005 12:38'!
additionsToViewerCategories
	^	#(
			(#'gps'
				(
					(command connect 'Connect to the GPS')
					(command disconnect 'Disconnect from the GPS')
					(slot isConnected 'Whether the GPS is connected' Boolean readOnly	Player getIsConnected unused unused)

					(slot latitude	'The latitude.'	Number readOnly Player getLatitude unused unused)
					(slot longitude	'The latitude.'	Number readOnly Player getLongitude unused unused)
					(slot altitude	'The Altitude.'	Number readOnly Player getAltitude unused unused)

					(slot course	'The Course.'	Number readOnly Player getCourse unused unused)
					(slot speed	'The Speed.'	Number readOnly Player getSpeed unused unused)

					(slot port	'The serial port where the GPS is connected.'	Number readWrite Player getPort Player setPort:)
				)
			)
		)
! !

!GPSMorph class methodsFor: 'private - forms' stamp: 'dgd 7/14/2005 13:22'!
connectedForm
"
self connectedForm asMorph openInHand
"
	^ ConnectedForm
		ifNil: [ConnectedForm := Form
						fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: 'R0lGODlhZABdALMAAEj++ww+IQzRCrWx/fXx+P+Wlv9OTh9ehdbS+bj/+f/Kyn9+tK2l/wD/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' readStream)]! !

!GPSMorph class methodsFor: 'parts bin' stamp: 'dgd 7/14/2005 12:59'!
descriptionForPartsBin
	^ self
		partName: 'GPS'
		categories: #('Multimedia' )
		documentation: 'An object to connect to a GPS'! !

!GPSMorph class methodsFor: 'private - forms' stamp: 'dgd 7/14/2005 13:22'!
disconnectedForm
"
	self disconnectedForm asMorph openInHand
"
	^ DisconnectedForm
		ifNil: [DisconnectedForm := Form
						fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: 'R0lGODlhZABdAOMPAEj++ww+IQzRCrWx/fXx+P+Wlv9OTh9ehdbS+bj/+f/Kyn9+tK2l/wD/
/wAAAP///yH5BAEAAA8ALAAAAABkAF0AAAT+8MlJq7046827/2AojmRpnmiqrmzrvnAsz3Rt
33iu73zv/8CgcEgsGo/IpHLJbDqf0Kh0Sq3uCA8CYrsdIAhgrLWzXSwOgYBjzQ4cFluruAJG
HNj4vD6wmEP5W2FneGluBw2IiQ0HaGprAQN+TW2DbQGKmJmIaGwHCFB6bJqjCRMJCQANjgGf
TQR5jo+Ho4kApwlhCYuPTQgLeLN3bbOapVm3uLrCkkW+ocSqsNC1yKdgiA4LRwS/a4ePbt6Z
wrCItgTV6LsDRQOriWuJwtOJsY+JYMjqDdlDWo7QxCECSIvTvQYAHugjgC0Asyvd3GASOFCA
REUAMsqLh0jhrX3+DlgB4XZwYkhFaCweAMCypctUxAjYOqUQm4NWPUie1FRyUwCLL4OyxPjA
VlGb7HxEpOXAIkYAaQSsFOoSU1Esm9ZouwKmZ6amlxC6TONgKtVUT4/yW5PURp0BcBnsZKpS
6M+yZ4cSzboG5wwtcRl4SUMLXt2gBwQIwHsWk95VD1kAZkCZcqQ75gqDNfvypxrOVUeR6yOD
wADLlQUPiHdKs8UAVKN+FioaT9sXpgWnhmtnTUK0o5ouPiubceivbSKfmJw6tTMHDbAA2PdV
sYO8jSwa12TPgXITWpo393LnUpbfthwLNy70jnW83PXAQKBbvGUEai7lK4pL/V3QQan+8d40
BgHzAgKn2VfZYGVFZw1NwCEU0mKw5QWVZ3PtskckLeSm4IL4NYiOProoMmFIFgLQTYbk5MEA
gl+skOCH9+XnYDVqNURhigA0okh3bMCRoBcpeFjfaUMK1pt+1TyIFoq+NVZbKEIO6cV3ZKBG
oxfdRDdiOjWhOFtQBYXiDYIzLoilBkbSaNlqKOZTjV4FrlSYPEAG2eZ4MYpgWppbhghdQl9a
c8wmaNAzTp6PCClemlf6CSikqJEnznTH3NLSnZi0GEoALwbmJocgIOgmn6scck4FJRbWiJl5
AAaogkiuSYGpp4K4YkjEbHpngZZo580dcOS6oGC2SrDnqZb+SqPoIq8S8pNi7/FhSDb0Gbvb
B8syO0CLhcBaCLXUkqXHG2fc8aK2u/m1way5ctndtOTWq1i4sGbDACMOiMoush50y+wX3+ab
r7lmroLrv5bZKnC8XtAXbUhleUrvvcPAUYmjDFfmbga4Jpnkkbvx9sUWajBwLR72xqIsF4ws
3DHAHDz8L1wkpWuxuHCg2QW82p72MQYyzwxinQ/8giEsq3gBdMc4e0Df00HzdkYaBMh18SP1
ssGAzTcPfQHY48Fltr9XgqG0AAlQC87FAPlrtGXcUl02F3j/jPPaqGDMNdx9yW30bRwUTWMj
jDBihmpfyNXUKeQGW+9nhhsdQuX+j+5qsAC2lBvLvRdrdwDZxoqAebvuGDxh335L3nW/dmtL
QuwNa85y555LG7rfV6JJu30k/OktfUD+xLrbukf1Oq8LRHz6hyWQrmRvLLNt72KuK9/6MF/L
GjvhpiOpZX336bTG9UC+pr2w+R4wvdkkIyk2t8LbBynx4KAvbe6qS9M80KRazvPehKB5aS8W
COtfwt4gOCKtoH6BQoACJ/gpBgZCcDR7IJpGNQDbUXCBC5ie7+AVNdxkK1Aq+6CzlFQH35HP
fskqwQY/dCVPHcwQL8qC9xgWQ/DM8FFKsuEw3vAivGGQXeArzZ/EV7Ln8Mp9eTsbE2dWwhzI
6lFOi8QzW842NyzOz4pLPFYXtxSpbVzxd0hMmytayEWSxU9LSOINGKoQBhdCLG09HIMe98jH
DkQAADs=' readStream)]! !

!GPSMorph class methodsFor: 'instance creation' stamp: 'dgd 7/14/2005 12:52'!
onPort: anInteger 
	"answer an instance of the receiver on a given port"
	^ self new initializeOnPort: anInteger! !

!GPSMorph methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:47'!
altitude
	^ self isConnected
		ifTrue: [gps altitude ]
		ifFalse: [0.0]! !

!GPSMorph methodsFor: 'connection' stamp: 'dgd 7/14/2005 12:46'!
connect
	self isConnected
		ifTrue: [^ self].
	""
	self disconnect.
	gps := GPS onPort: port.
	gps isConnected
		ifFalse: [self disconnect]! !

!GPSMorph methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:47'!
course
	^ self isConnected
		ifTrue: [gps course degrees]
		ifFalse: [0.0]! !

!GPSMorph methodsFor: 'scripting' stamp: 'dgd 7/14/2005 13:02'!
defaultFloatPrecisionFor: aGetSelector 
	"Answer a number indicating the default float precision to be  
	used in a numeric readout for which the receiver provides the 
	data. Individual morphs can override this. Showing fractional  
	values for readouts of getCursor was in response to an explicit 
	request from ack"
	(#(#getSpeed  ) includes: aGetSelector)
		ifTrue: [^ 0.1].
	(#(#getAltitude ) includes: aGetSelector)
		ifTrue: [^ 0.01].
	(#(#getLongitude #getLatitude #getCourse ) includes: aGetSelector)
		ifTrue: [^ 0.001].
	^ super defaultFloatPrecisionFor: aGetSelector! !

!GPSMorph methodsFor: 'submorphs-add/remove' stamp: 'dgd 7/14/2005 12:54'!
delete
self disconnect.
super delete! !

!GPSMorph methodsFor: 'connection' stamp: 'dgd 7/14/2005 12:45'!
disconnect
	"try to disconnect as gently as possible"
	gps isNil
		ifTrue: [^ self].
	""
	gps disconnect.
	gps := nil! !

!GPSMorph methodsFor: 'drawing' stamp: 'dgd 7/14/2005 13:32'!
drawOn: aCanvas 
	| formToDraw |
	super drawOn: aCanvas.
	""
formToDraw := self isConnected
				ifTrue: [self class connectedForm]
				ifFalse: [self class disconnectedForm].
	aCanvas
		translucentImage: (formToDraw)
		at: self innerBounds origin! !

!GPSMorph methodsFor: 'geometry' stamp: 'dgd 7/14/2005 13:24'!
extent: aPoint 
	super extent:self fixedExtent! !

!GPSMorph methodsFor: 'geometry' stamp: 'dgd 7/14/2005 13:31'!
fixedExtent
	^ (self class connectedForm extent max: self class disconnectedForm extent)
		+ (self borderWidth * 2)

! !

!GPSMorph methodsFor: 'initialization' stamp: 'dgd 7/14/2005 14:12'!
initialize
	super initialize.
	self extent: 10 @ 10.
	""
	self color: Color red muchLighter.
	self extent: 200 @ 300.
	self useRoundedCorners.
	self borderWidth: 2.
	self borderColor: self color muchDarker.
	""
	port := 0! !

!GPSMorph methodsFor: 'initialization' stamp: 'dgd 7/14/2005 12:52'!
initializeOnPort: anInteger 
	"initialize the receiver on a given port"
	port := anInteger.
	self connect! !

!GPSMorph methodsFor: 'connection' stamp: 'dgd 7/14/2005 12:34'!
isConnected
	^ gps notNil
		and: [gps isConnected]! !

!GPSMorph methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:43'!
latitude
	^ self isConnected ifTrue:[gps latitude degrees] ifFalse:[0.0]! !

!GPSMorph methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:47'!
longitude
	^ self isConnected
		ifTrue: [gps longitude degrees]
		ifFalse: [0.0]! !

!GPSMorph methodsFor: 'halos and balloon help' stamp: 'dgd 7/14/2005 12:32'!
okayToResizeEasily
	"Answer whether it is appropriate to have the receiver be easily 
	resized by the user from the halo"
	^ false! !

!GPSMorph methodsFor: 'connection' stamp: 'dgd 7/14/2005 12:41'!
port
	^ port
		ifNil: [0] ! !

!GPSMorph methodsFor: 'connection' stamp: 'dgd 7/14/2005 12:57'!
port: aNumber 
	| newPort |
	newPort := aNumber asInteger max: 0.
	newPort = port
		ifFalse: [""

			self disconnect.
			port := newPort.
			self connect]! !

!GPSMorph methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:47'!
speed
	^ self isConnected
		ifTrue: [gps speed ]
		ifFalse: [0.0]! !

!GPSMorph methodsFor: 'stepping and presenter' stamp: 'dgd 7/14/2005 13:29'!
step
	self isConnected
		ifTrue: [self color: Color green muchLighter]
		ifFalse: [self color: Color red muchLighter].
	self borderColor: self color muchDarker.! !

SerialPort subclass: #GPS
	instanceVariableNames: 'portNumber readerProcess buffer sleeping rest utcTime latitude longitude altitude satellites fixMode fix positionDilutionOfPrecision horizontalDilutionOfPrecision verticalDilutionOfPrecision course speed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Core'!

!GPS class methodsFor: 'class initialization' stamp: 'dgd 7/5/2005 13:51'!
initialize
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!GPS class methodsFor: 'instance creation' stamp: 'dgd 7/5/2005 13:51'!
onPort: anInteger 
	"answer an instance of the receiver on a given port"
	^ self new initializeOnPort: anInteger! !

!GPS class methodsFor: 'system startup' stamp: 'dgd 7/6/2005 11:02'!
shutDown: resuming 
	resuming
		ifTrue: [^ self].""
	self allSubInstances
		do: [:each | each sleep]! !

!GPS class methodsFor: 'system startup' stamp: 'dgd 7/6/2005 11:03'!
startUp: resuming 
	resuming
		ifTrue: [^ self].
	""
	self allSubInstances
		do: [:each | each awake]! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:49'!
altitude
	^ altitude! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/13/2005 12:48'!
altitude: altitudeString
	"private - update the receiver's altitude and altitudeUnits"
	altitudeString asNumber isZero
		ifFalse: [altitude := altitudeString asNumber]! !

!GPS methodsFor: 'sleeping' stamp: 'dgd 7/14/2005 11:31'!
awake
"riiinggg!!!!!! time to awake"
	sleeping
		ifTrue: [""
			self connect.
			sleeping := false]! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 12:42'!
checkSumFor: lineString 
"private - answer the NMEA checksum for the given string"
	| checkSum |
	checkSum := 0.
	(lineString allButFirst allButLast: 3)
		do: [:each | checkSum := checkSum bitXor: each asciiValue].
	^ checkSum hex allButFirst: 3! !

!GPS methodsFor: 'connecting' stamp: 'dgd 7/14/2005 11:29'!
connect
"connect the receiver to the real GPS"
	self disconnect.
	""
	self openPort: portNumber.
	self startReading! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:48'!
course
	^ course! !

!GPS methodsFor: 'connecting' stamp: 'dgd 7/14/2005 11:29'!
disconnect
"disconnect the receiver from the real GPS"
	self stopReading.
	self close.
	self initialize! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 11:38'!
fillBuffer
	"private - fill the buffer with the available data"
	| data |
	sleeping
		ifTrue: [^ self].
	""
	[data := super readString.
	(data notNil
			and: [data notEmpty])
		ifTrue: [buffer nextPut: data]]
		on: Error
		do: [:e | 
			self flag: #todo.
			"flag the instance as 'poison' to try to reconnect in background"
			self close]! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 11:46'!
fillBufferLoop
	"private - fill the buffer, wait, fill the buffer, wait,........."
	[self fillBuffer.
self processBuffer.
	(Delay forMilliseconds: 50) wait] repeat! !

!GPS methodsFor: 'input/output' stamp: 'dgd 7/6/2005 11:13'!
flush
"flush the buffer"
	buffer flush! !

!GPS methodsFor: 'initialization' stamp: 'dgd 7/14/2005 11:43'!
initialize
	"initialize the receiver"
	super initialize.
	""
	utcTime := Time
				hour: 0
				minute: 0
				second: 0.
	""
	latitude := GPSLatitude degrees: 0.
	longitude := GPSLongitude degrees: 0.0.
	altitude := 0.0.
	""
	satellites := OrderedCollection new.
	""
	fixMode := 'A'.
	fix := 1.
	""
	positionDilutionOfPrecision := 0.
	horizontalDilutionOfPrecision := 0.
	verticalDilutionOfPrecision := 0.
""
course := GPSAngle degrees:0.
	speed := 0.0.
	""
	rest := ''.
	sleeping := false.
	buffer := SharedQueue new.
	""
	self baudRate: 9600.
	self dataBits: 8.
	self parityType: 0.
	self stopBitsType: 1! !

!GPS methodsFor: 'initialization' stamp: 'dgd 7/14/2005 11:28'!
initializeOnPort: anInteger 
	"initialize the receiver on a given port"
	portNumber := anInteger.
	self connect! !

!GPS methodsFor: 'connecting' stamp: 'dgd 7/14/2005 11:30'!
isConnected
	"answer if the receiver is connected to the real GPS"
	^ port notNil! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:44'!
latitude
	^ latitude! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:48'!
longitude
	^ longitude! !

!GPS methodsFor: 'printing' stamp: 'dgd 7/14/2005 11:44'!
printOn: aStream 
	aStream nextPutAll: 'GPS #' , portNumber asString , ': ';
		 nextPutAll: 'UTC Time: ';
		 nextPutAll: utcTime asString;
		 nextPutAll: ', Latitude: ';
		 nextPutAll: latitude asString;
		 nextPutAll: ', Longitude: ';
		 nextPutAll: longitude asString;
		 nextPutAll: ', Altitude: ';
		 nextPutAll: altitude asString;
		 nextPutAll: ', Course: ';
		 nextPutAll: course asString;
		 nextPutAll: ', Speed: ';
		 nextPutAll: speed asString;
		 nextPutAll: ', Satellites: ';
		 nextPutAll: satellites size asString.
	""
	aStream
		nextPutAll: (self isConnected
				ifTrue: [' (Connected)']
				ifFalse: [' (**DISCONNECTED**)'])! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 11:54'!
processBuffer
	| item data |
	item := buffer nextOrNil.
	item isNil
		ifTrue: [^ self].
	""
	data := String new writeStream.
	[item notNil]
		whileTrue: [""
			data nextPutAll: item.
			item := buffer nextOrNil].
	""
	self processData: data contents! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 12:17'!
processData: rawDataString 
	(rawDataString includes: Character lf)
		ifTrue: [| lines | 
			lines := rest
						, (rawDataString copyUpToLast: Character lf) findTokens: Character cr asString , Character lf asString.
			rest := rawDataString copyAfterLast: Character lf.
			lines
				do: [:line | self processLine: line]]
		ifFalse: [rest := rest , rawDataString] ! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/13/2005 12:48'!
processGPGGA: tokensCollection 
	"private - $GPGGA Global positioning system fixed data"
	self
		utcTime: tokensCollection second
		latitude: tokensCollection third
		nsIndicator: tokensCollection fourth
		longitude: tokensCollection fifth
		ewIndicator: tokensCollection sixth.
	self
		altitude: (tokensCollection at: 10)
	"altitudeUnits: (tokensCollection at: 11)"! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/6/2005 19:26'!
processGPGLL: tokensCollection 
	"private - $GPGLL Geographic position - latitude / longitude"
	self utcTime: tokensCollection sixth latitude: tokensCollection second nsIndicator: tokensCollection third longitude: tokensCollection fourth ewIndicator: tokensCollection fifth! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/7/2005 11:57'!
processGPGSA: tokensCollection 
	"private - $GPGSA	GNSS DOP and active satellites"
	fixMode := tokensCollection second.
	fix := tokensCollection third asNumber.
	""
	positionDilutionOfPrecision := (tokensCollection at: 16) asNumber.
	horizontalDilutionOfPrecision := (tokensCollection at: 17) asNumber.
	verticalDilutionOfPrecision := (tokensCollection at: 18) asNumber! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/7/2005 12:09'!
processGPGSV: tokensCollection 
	"private - $GPGSV	GNSS satellites in view"
	"id.elevation.azimuth.snr."
	tokensCollection third asNumber = 1
		ifTrue: [satellites := OrderedCollection new].
	1
		to: 4
		do: [:index | 
			| id | 
			id := tokensCollection at: index * 4 + 1.
			id isEmpty
				ifFalse: [| elevation azimuth signalToNoiseRation | 
					elevation := tokensCollection at: index * 4 + 2 ifAbsent: ['0'].
					azimuth := tokensCollection at: index * 4 + 3 ifAbsent: ['0'].
					signalToNoiseRation := tokensCollection at: index * 4 + 4 ifAbsent: ['0'].
					""
					satellites
						removeAllSuchThat: [:each | each id = id].
					satellites
						add: (GPSSatellite
								id: id
								elevation: elevation
								azimuth: azimuth
								signalToNoiseRation: signalToNoiseRation)]]! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/6/2005 19:26'!
processGPRMC: tokensCollection 
	"private - $GPRMC Recommended minimum specific GNSS  
	data"
	self utcTime: tokensCollection second latitude: tokensCollection fourth nsIndicator: tokensCollection fifth longitude: tokensCollection sixth ewIndicator: tokensCollection seventh! !

!GPS methodsFor: 'private - nmea sentences' stamp: 'dgd 7/14/2005 11:43'!
processGPVTG: tokensCollection 
	"private - $GPVTG	Course over ground and ground speed"
	" 
	Field	Example	Comments  
	Sentence ID	$GPVTG	  
	Course	89.68	Course in degrees  
	Reference	89.68	T = True heading  
	Course	 Course in degrees  
	Reference	89.68	M = Magnetic heading  
	Speed	0.00	Horizontal speed  
	Units	N	N = Knots  
	Speed	0.00	Horizontal speed  
	Units	K	K = KM/h  
	"
	course := GPSAngle fromString: tokensCollection second.
	speed := tokensCollection eighth asNumber.
! !

!GPS methodsFor: 'private' stamp: 'dgd 7/7/2005 11:13'!
processLine: lineString 
	| tokens |
	(self validateLine: lineString)
		ifFalse: [^ self].
	""
	tokens := (lineString allButLast: 3) commaDelimitedFields.
	tokens first = '$GPGGA'
		ifTrue: [^ self processGPGGA: tokens].
	tokens first = '$GPGLL'
		ifTrue: [^ self processGPGLL: tokens].
	tokens first = '$GPRMC'
		ifTrue: [^ self processGPRMC: tokens].
	tokens first = '$GPGSA'
		ifTrue: [^ self processGPGSA: tokens].
	tokens first = '$GPGSV'
		ifTrue: [^ self processGPGSV: tokens].
	tokens first = '$GPVTG'
		ifTrue: [^ self processGPVTG: tokens].
	""
	Transcript show: tokens first;
		 cr! !

!GPS methodsFor: 'sleeping' stamp: 'dgd 7/14/2005 11:31'!
sleep
	"put the receiver to sleep"
	self isConnected
		ifTrue: [sleeping := true].
	self close! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 12:48'!
speed
	^ speed! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 11:17'!
startReading
	readerProcess := [
			self fillBufferLoop]
				forkAt: Processor userInterruptPriority! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 11:30'!
stopReading
	readerProcess isNil ifTrue:[^ self].

""
			readerProcess terminate.
			readerProcess := nil! !

!GPS methodsFor: 'accessing' stamp: 'dgd 7/14/2005 11:29'!
utcTime: utcTimeString latitude: latitudeString nsIndicator: nsIndicatorString longitude: longitudeString ewIndicator: ewIndicatorString 
	"private - change the receiver's utcTime, latitude and  
	longitude"
	utcTime := Time
				hour: (utcTimeString first: 2) asNumber
				minute: (utcTimeString copyFrom: 3 to: 4)
				second: (utcTimeString copyFrom: 5 to: 6)
				nanoSecond: (utcTimeString last: 3) asNumber * 1000.
	latitude := GPSLatitude fromString: latitudeString nsIndicator: nsIndicatorString.
	longitude := GPSLongitude fromString: longitudeString ewIndicator: ewIndicatorString! !

!GPS methodsFor: 'private' stamp: 'dgd 7/6/2005 12:53'!
validateLine: lineString 
	"private - answer if the given string is a valid NMEA line"
	lineString first = $$
		ifFalse: [^ false].
	""
	(lineString last: 3) first = $*
		ifFalse: [^ false].
	""
	(lineString last: 2) = (self checkSumFor: lineString)
		ifFalse: [^ false].
	""
	^ true! !

!Player methodsFor: '*gps-connection' stamp: 'dgd 7/14/2005 11:49'!
connect
	self sendMessageToCostume: #connect! !

!Player methodsFor: '*gps-connection' stamp: 'dgd 7/14/2005 11:49'!
disconnect
		self sendMessageToCostume: #disconnect! !

!Player methodsFor: '*gps-accessing' stamp: 'dgd 7/14/2005 12:02'!
getAltitude
	^ self sendMessageToCostume: #altitude! !

!Player methodsFor: '*gps-accessing' stamp: 'dgd 7/14/2005 12:02'!
getCourse
	^ self sendMessageToCostume: #course! !

!Player methodsFor: '*gps-connection' stamp: 'dgd 7/14/2005 11:53'!
getIsConnected
	^ self sendMessageToCostume: #isConnected! !

!Player methodsFor: '*gps-accessing' stamp: 'dgd 7/14/2005 12:01'!
getLatitude
	^ self sendMessageToCostume: #latitude! !

!Player methodsFor: '*gps-accessing' stamp: 'dgd 7/14/2005 12:02'!
getLongitude
	^ self sendMessageToCostume: #longitude! !

!Player methodsFor: '*gps-connection' stamp: 'dgd 7/14/2005 12:41'!
getPort
	^ self sendMessageToCostume: #port! !

!Player methodsFor: '*gps-accessing' stamp: 'dgd 7/14/2005 12:02'!
getSpeed
	^ self sendMessageToCostume: #speed! !

!Player methodsFor: '*gps-connection' stamp: 'dgd 7/14/2005 12:41'!
setPort: aNumber 
	^ self sendMessageToCostume: #port: with: aNumber! !

!AbstractString methodsFor: '*gps-accessing' stamp: 'dgd 7/6/2005 21:03'!
commaDelimitedFields
"Considering the receiver as a holder of comma-delimited  
	fields and answer a collection of them"
	| result |
	result := OrderedCollection new.
	self
		commaDelimitedFieldsDo: [:field | result add: field].
	^ result! !

!AbstractString methodsFor: '*gps-accessing' stamp: 'dgd 7/6/2005 20:54'!
commaDelimitedFieldsDo: aBlock 
	"Considering the receiver as a holder of comma-delimited  
	fields, evaluate execute aBlock with each field in this string.  
	The separating commas are not included in what is passed to  
	aBlock"
			" 
			'fred,charlie,elmo,,2' commaDelimitedFieldsDo: [:aField | Transcript show: aField; cr] 
			"
	| start end |
	start := 1.
	[start <= self size]
		whileTrue: [""
			end := self
						indexOf: $,
						startingAt: start
						ifAbsent: [self size + 1].
			end := end - 1.
			aBlock
				value: (self copyFrom: start to: end).
			start := end + 2
]! !

TestCase subclass: #GPSTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GPS-Test'!

!GPSTest methodsFor: 'running' stamp: 'dgd 7/7/2005 13:23'!
testAngle
	self
		should: [(GPSAngle degrees: 90)
				= (GPSAngle radians: Float halfPi)]
! !

!GPSTest methodsFor: 'running' stamp: 'dgd 7/11/2005 12:08'!
testAngleNormalize
	self
		should: [(GPSAngle degrees: 90)
				= (GPSAngle degrees: 450)]! !
GPS initialize!
